// THE GAME ITSELF

import { GameOptions } from './gameOptions';
import { Levels } from './levels';

// this class extends Scene class
export class PlayGame extends Phaser.Scene {

    constructor() {
        super({
            key : 'PlayGame'
        });
    }

    level : number;
    floorLevel : number;
    theSquare : Phaser.Physics.Arcade.Sprite;
    canJump : boolean;
    backgroundGroup : Phaser.GameObjects.Group;
    groundGroup : Phaser.Physics.Arcade.Group;
    obstacleGroup : Phaser.Physics.Arcade.Group;
    jumpTween : Phaser.Tweens.Tween;
    emitter : Phaser.GameObjects.Particles.ParticleEmitter;

    // method to be called once the instance has been created
    create() : void {
        this.backgroundGroup = this.add.group();
        this.groundGroup = this.physics.add.group();
        this.obstacleGroup = this.physics.add.group();
        this.level = 0;
        this.floorLevel = 0;
        this.canJump = true;
        this.drawLevel();
        this.emitter = this.add.particles(0, 0, 'tile', {
            gravityY : 20,
            speed : {
                min : 20,
                max : 50
            },
            scale : {
                min : 0.05,
                max : 0.1
            },
            lifespan : 800,
            alpha : {
                start : 1,
                end: 0
            },
            emitting : false
        });
        this.emitter.setDepth(2);
        this.theSquare = this.physics.add.sprite(0, 0, 'tile');
        this.theSquare.setDepth(1);
        this.placeSquare();
        this.input.on('pointerdown', this.squareJump, this);  
    }

    drawLevel() : void {
        this.backgroundGroup.clear(true, true);
        this.groundGroup.clear(true, true);
        this.obstacleGroup.clear(true, true);
        let floorHeight : number = GameOptions.level.height / Levels[this.level].floors.length;
        for (let i : number = 0; i < Levels[this.level].floors.length; i ++) {
            let background : Phaser.GameObjects.TileSprite = this.add.tileSprite(0, floorHeight * i, GameOptions.level.width, floorHeight, 'tile');
            background.setTint(Levels[this.level].floors[i].colors.background);    
            background.setOrigin(0);
            this.backgroundGroup.add(background);
            let floor : Phaser.GameObjects.TileSprite = this.add.tileSprite(- Levels[this.level].floors[this.floorLevel].square.size, floorHeight * i + floorHeight, GameOptions.level.width + 2 * Levels[this.level].floors[this.floorLevel].square.size, 16, 'tile');    
            floor.setOrigin(0);
            this.physics.add.existing(floor);
            this.groundGroup.add(floor);
            // @ts-ignore
            floor.body.pushable = false;
            if (Levels[this.level].floors[i].obstacles) {
                for (let j : number = 0; j < Levels[this.level].floors[i].obstacles.length; j ++) {  
                    let obstacleX : number = (i % 2 == 0) ? Levels[this.level].floors[i].obstacles[j].start : this.game.config.width as number - Levels[this.level].floors[i].obstacles[j].start - Levels[this.level].floors[i].obstacles[j].width;
                    let obstacleY : number = floorHeight * i + floorHeight - Levels[this.level].floors[i].obstacles[j].ground - Levels[this.level].floors[i].obstacles[j].height;
                    let spike : Phaser.GameObjects.TileSprite = this.add.tileSprite(obstacleX, obstacleY, Levels[this.level].floors[i].obstacles[j].width, Levels[this.level].floors[i].obstacles[j].height, 'tile');
                    spike.setTint(Levels[this.level].floors[i].colors.foreground); 
                    spike.setOrigin(0);
                    this.physics.add.existing(spike);
                    // @ts-ignore
                    spike.body.pushable = false;
                    this.obstacleGroup.add(spike);         
                }
            }
        }
    }

    placeSquare() : void {
        if (this.jumpTween) {
            this.jumpTween.stop();
            this.theSquare.angle = 0;
        }    
        this.theSquare.displayWidth = Levels[this.level].floors[this.floorLevel].square.size;
        this.theSquare.displayHeight = Levels[this.level].floors[this.floorLevel].square.size;
        this.theSquare.setGravityY(Levels[this.level].floors[this.floorLevel].square.gravity);
        this.theSquare.setTint(Levels[this.level].floors[this.floorLevel].colors.foreground);  
        this.theSquare.setVelocityX((this.floorLevel % 2 == 0) ? Levels[this.level].floors[this.floorLevel].square.speed : - Levels[this.level].floors[this.floorLevel].square.speed);
        this.theSquare.setVelocityY(0);
        this.theSquare.setX((this.floorLevel % 2 == 0) ? 0 - Levels[this.level].floors[this.floorLevel].square.size : GameOptions.level.width + Levels[this.level].floors[this.floorLevel].square.size);
        this.theSquare.setY(GameOptions.level.height / Levels[this.level].floors.length * (this.floorLevel + 1) - Levels[this.level].floors[this.floorLevel].square.size / 2);
        this.theSquare.setData('maxY', this.theSquare.y);
        this.canJump = true; 
    }

    squareJump() : void {
        if (this.canJump) {
            this.canJump = false;
            this.theSquare.setVelocityY(Levels[this.level].floors[this.floorLevel].square.jumpForce * -1);
            let jumpAngle : number = this.floorLevel % 2 == 0 ? 180 : -180;
            this.jumpTween = this.tweens.add({
                targets : this.theSquare,
                angle : this.theSquare.angle + jumpAngle,
                duration : Levels[this.level].floors[this.floorLevel].square.jumpForce / Levels[this.level].floors[this.floorLevel].square.gravity * 2000
            })
        } 
    }

    handleCollision() : void {
        this.emitter.x = this.theSquare.x;
        this.emitter.y = this.theSquare.y;
        this.emitter.explode(32);
        this.emitter.forEachAlive((particle : Phaser.GameObjects.Particles.Particle) => {
            particle.tint = this.theSquare.tintTopLeft;
        }, this);   
        this.placeSquare();
    }

    update() : void {
        console.log(this.theSquare.getData('maxY'))
        if (this.theSquare.y > this.theSquare.getData('maxY')) {
            this.theSquare.y = this.theSquare.getData('maxY');
        }
        this.physics.collide(this.theSquare, this.groundGroup);
        this.physics.overlap(this.theSquare, this.obstacleGroup, this.handleCollision, undefined, this);
        // @ts-ignore
        if (this.theSquare.body.touching.down) {
            this.canJump = true;
        }
        if ((this.theSquare.x > GameOptions.level.width && this.floorLevel % 2 == 0) || (this.theSquare.x < 0 && this.floorLevel % 2 == 1)) {
            this.floorLevel ++;
            if (this.floorLevel == Levels[this.level].floors.length) {
                this.floorLevel = 0;
                this.level ++;
                if (this.level == Levels.length) {
                    this.level = 0;
                }
                this.drawLevel();
            }
            this.placeSquare();
        }
    }
}